
import numpy as np,pandas as pd
from sklearn.preprocessing import MinMaxScaler
from eviboost import *
from tir import TIR,selectq

"""
Estimate the tail index of four commercial banks separately.
We do min-max transformation on x using sklearn.preprocessing.MinMaxScaler.
The optimal fraction q_opt is determined by selectq().
"""

dataset = 'bank_data.csv' 
d = pd.read_csv(dataset)
d = d[d['tic']=='...'] # BAC,JPM,WFC,C

y = d['ret'].values
x = d.values[:,3:].astype(float)
y = np.abs(y)+(1e-20)
n,p = x.shape
th0 = np.full(p+1,0.5)

minmax = MinMaxScaler()
xx = minmax.fit_transform(x)

DS = selectq(xx,y)
(f1,f,d1) = DS[0]
(f2,f,d2) = DS[1]
(f3,f,d3) = DS[2]
print('argmin D1,D2,D3=',f1,f2,f3)


"""
Fit {xx,y} by EVIboost. Model parameters are shown below (copied from Table2 in our paper).
-------------------------------------
dataset  q_opt(%)  D   V       M 
BAC      7.5       1   0.005   400
C        16.0      1   0.0075  160
JPM      5.0       1   0.0075  95
WFC      4.5       1   0.01    30
-------------------------------------
Use the function adjust_imp(), partial() to compute adjusted feature importance measures and the partial dependence of x_j.
"""

q_opt,D,V,M = ...
model = EVIboost()
model.fit(xx,y,q_opt,n_estimators=M,learning_rate=V,max_depth=D)
gamma_hat = model.predict(xx)


def adjust_imp(iterations=100):
    np.random.seed(123)
    imp = np.zeros(p)
    for r in range(iterations):
        x,xs = xx.copy(),xx.copy()
        np.random.shuffle(xs)
        X = np.hstack((x,xs))
        m = EVIboost()
        m.fit(X,y,q_opt,n_estimators=M,learning_rate=V,max_depth=D)
        I = m.feature_importance(2*p)
        I1 = I[:p]
        I2 = I[p:]
        imp += (I1-I2)
        print(r)
        
    imp = [abs(m) for m in imp]
    imp = imp/sum(imp)
    return imp

def partial(j):
    x0 = xx.copy()
    G = []
    for xj in np.linspace(0,1,100):
        x0[:,j] = xj
        G0 = np.mean(model.predict(x0))
        G.append(G0)
    return G

print(adjust_imp())
print(partial(j))




